@@ -320,6 +320,11 @@ impl Manager {
                     )
                 }
             }
+            Broadcast::Invite {
+                inviter,
+                channel,
+                user_channels,
+            } => message::broadcast::invite(inviter, channel, user_channels),
         };
 
         messages.into_iter().for_each(|message| {
@@ -519,4 +524,9 @@ pub enum Broadcast {
         ourself: bool,
         user_channels: Vec<String>,
     },
+    Invite {
+        inviter: Nick,
+        channel: String,
+        user_channels: Vec<String>,
+    },
 }
