@@ -43,6 +43,12 @@ pub struct State {
     input: String,
 }
 
+impl Default for State {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl State {
     pub fn new() -> Self {
         Self {
@@ -91,4 +97,14 @@ impl State {
     pub fn reset(&self) -> Command<Message> {
         input::reset(self.input_id.clone())
     }
+
+    pub fn insert_user(&mut self, user: User) -> Command<Message> {
+        if self.input.is_empty() {
+            self.input = format!("{}: ", user.nickname());
+        } else {
+            self.input = format!("{} {}", self.input, user.nickname());
+        }
+
+        input::move_cursor_to_end(self.input_id.clone())
+    }
 }
