@@ -1,7 +1,7 @@
 use data::message;
-use data::palette::{self, Palette};
+use data::theme::{randomize_color, Colors};
 use iced::widget::{button, container, pane_grid, rule, scrollable, text, text_input};
-use iced::{application, Background, Color};
+use iced::{application, overlay, Background, Color};
 
 use crate::widget::selectable_text;
 
@@ -10,10 +10,51 @@ use crate::widget::selectable_text;
 pub const TEXT_SIZE: f32 = 13.0;
 pub const ICON_SIZE: f32 = 12.0;
 
-#[derive(Clone)]
-pub struct Theme {
-    pub palette: Palette,
-    pub colors: Colors,
+#[derive(Debug, Clone)]
+pub enum Theme {
+    Selected(data::Theme),
+    Preview {
+        selected: data::Theme,
+        preview: data::Theme,
+    },
+}
+
+impl Theme {
+    pub fn preview(&self, theme: data::Theme) -> Self {
+        match self {
+            Theme::Selected(selected) | Theme::Preview { selected, .. } => Self::Preview {
+                selected: selected.clone(),
+                preview: theme,
+            },
+        }
+    }
+
+    pub fn selected(&self) -> Self {
+        match self {
+            Theme::Selected(selected) | Theme::Preview { selected, .. } => {
+                Self::Selected(selected.clone())
+            }
+        }
+    }
+
+    fn colors(&self) -> &Colors {
+        match self {
+            Theme::Selected(selected) => &selected.colors,
+            Theme::Preview { preview, .. } => &preview.colors,
+        }
+    }
+}
+
+impl From<data::Theme> for Theme {
+    fn from(theme: data::Theme) -> Self {
+        Theme::Selected(theme)
+    }
+}
+
+impl Default for Theme {
+    fn default() -> Self {
+        Self::from(data::Theme::default())
+    }
 }
 
 impl application::StyleSheet for Theme {
@@ -21,117 +62,12 @@ impl application::StyleSheet for Theme {
 
     fn appearance(&self, _style: &Self::Style) -> application::Appearance {
         application::Appearance {
-            background_color: self.colors.background.base,
-            text_color: self.colors.text.base,
+            background_color: self.colors().background.base,
+            text_color: self.colors().text.base,
         }
     }
 }
 
-impl Default for Theme {
-    fn default() -> Self {
-        let palette = Palette::default();
-
-        Theme {
-            palette,
-            colors: Colors::colors_from_palette(&palette),
-        }
-    }
-}
-
-impl Theme {
-    pub fn new_from_palette(palette: data::palette::Palette) -> Self {
-        Theme {
-            palette,
-            colors: Colors {
-                background: Subpalette::from_color(palette.background, &palette),
-                text: Subpalette::from_color(palette.text, &palette),
-                action: Subpalette::from_color(palette.action, &palette),
-                accent: Subpalette::from_color(palette.accent, &palette),
-                alert: Subpalette::from_color(palette.alert, &palette),
-                error: Subpalette::from_color(palette.error, &palette),
-                info: Subpalette::from_color(palette.info, &palette),
-                success: Subpalette::from_color(palette.success, &palette),
-            },
-        }
-    }
-}
-
-#[derive(Debug, Clone)]
-pub struct Colors {
-    pub background: Subpalette,
-    pub text: Subpalette,
-    pub action: Subpalette,
-    pub accent: Subpalette,
-    pub alert: Subpalette,
-    pub error: Subpalette,
-    pub info: Subpalette,
-    pub success: Subpalette,
-}
-
-impl Colors {
-    pub fn colors_from_palette(palette: &Palette) -> Self {
-        Colors {
-            background: Subpalette::from_color(palette.background, palette),
-            text: Subpalette::from_color(palette.text, palette),
-            action: Subpalette::from_color(palette.action, palette),
-            accent: Subpalette::from_color(palette.accent, palette),
-            alert: Subpalette::from_color(palette.alert, palette),
-            error: Subpalette::from_color(palette.error, palette),
-            info: Subpalette::from_color(palette.info, palette),
-            success: Subpalette::from_color(palette.success, palette),
-        }
-    }
-}
-
-#[derive(Debug, Clone)]
-pub struct Subpalette {
-    pub base: Color,
-    pub light: Color,
-    pub lighter: Color,
-    pub lightest: Color,
-    pub dark: Color,
-    pub darker: Color,
-    pub darkest: Color,
-    pub low_alpha: Color,
-    pub med_alpha: Color,
-    pub high_alpha: Color,
-}
-
-impl Subpalette {
-    pub fn from_color(color: Color, palette: &data::palette::Palette) -> Subpalette {
-        let is_dark = palette::is_dark(palette.background);
-
-        Subpalette {
-            base: color,
-            light: palette::lighten(color, 0.03),
-            lighter: palette::lighten(color, 0.06),
-            lightest: palette::lighten(color, 0.12),
-            dark: palette::darken(color, 0.03),
-            darker: palette::darken(color, 0.06),
-            darkest: palette::darken(color, 0.12),
-            low_alpha: if is_dark {
-                palette::alpha(color, 0.4)
-            } else {
-                palette::alpha(color, 0.8)
-            },
-            med_alpha: if is_dark {
-                palette::alpha(color, 0.2)
-            } else {
-                palette::alpha(color, 0.4)
-            },
-            high_alpha: if is_dark {
-                palette::alpha(color, 0.1)
-            } else {
-                palette::alpha(color, 0.3)
-            },
-        }
-    }
-
-    pub fn is_dark(&self) -> bool {
-        palette::is_dark(self.base)
-    }
-}
-
 #[derive(Debug, Clone, Default)]
 pub enum Rule {
     #[default]
@@ -145,13 +81,13 @@ impl rule::StyleSheet for Theme {
     fn appearance(&self, style: &Self::Style) -> rule::Appearance {
         match style {
             Rule::Default => rule::Appearance {
-                color: self.colors.background.light,
+                color: self.colors().background.light,
                 width: 1,
                 radius: 0.0.into(),
                 fill_mode: rule::FillMode::Full,
             },
             Rule::Unread => rule::Appearance {
-                color: self.colors.accent.base,
+                color: self.colors().accent.base,
                 width: 1,
                 radius: 0.0.into(),
                 fill_mode: rule::FillMode::Full,
@@ -182,39 +118,39 @@ impl text::StyleSheet for Theme {
         match style {
             Text::Default => text::Appearance { color: None },
             Text::Primary => text::Appearance {
-                color: Some(self.colors.text.base),
+                color: Some(self.colors().text.base),
             },
             Text::Action => text::Appearance {
-                color: Some(self.colors.action.base),
+                color: Some(self.colors().action.base),
             },
             Text::Accent => text::Appearance {
-                color: Some(self.colors.accent.base),
+                color: Some(self.colors().accent.base),
             },
             Text::Info => text::Appearance {
-                color: Some(self.colors.info.base),
+                color: Some(self.colors().info.base),
             },
             Text::Error => text::Appearance {
-                color: Some(self.colors.error.base),
+                color: Some(self.colors().error.base),
             },
             Text::Nickname(seed) => {
-                let original_color = self.colors.action.base;
+                let original_color = self.colors().action.base;
                 let color = seed
-                    .map(|seed| palette::randomize_color(original_color, seed.as_str()))
+                    .map(|seed| randomize_color(original_color, seed.as_str()))
                     .unwrap_or_else(|| original_color);
 
                 text::Appearance { color: Some(color) }
             }
             Text::Server => text::Appearance {
-                color: Some(self.colors.info.base),
+                color: Some(self.colors().info.base),
             },
             Text::Status(status) => text::Appearance {
                 color: Some(match status {
-                    message::source::Status::Success => self.colors.success.base,
-                    message::source::Status::Error => self.colors.error.base,
+                    message::source::Status::Success => self.colors().success.base,
+                    message::source::Status::Error => self.colors().error.base,
                 }),
             },
             Text::Transparent => text::Appearance {
-                color: Some(self.colors.text.low_alpha),
+                color: Some(self.colors().text.low_alpha),
             },
         }
     }
@@ -234,6 +170,7 @@ pub enum Container {
     },
     Context,
     Highlight,
+    SemiTransparent,
 }
 
 impl container::StyleSheet for Theme {
@@ -245,30 +182,30 @@ impl container::StyleSheet for Theme {
                 ..Default::default()
             },
             Container::Primary => container::Appearance {
-                background: Some(Background::Color(self.colors.background.base)),
-                text_color: Some(self.colors.text.base),
+                background: Some(Background::Color(self.colors().background.base)),
+                text_color: Some(self.colors().text.base),
                 ..Default::default()
             },
             Container::PaneBody { selected } => container::Appearance {
-                background: Some(Background::Color(self.colors.background.dark)),
+                background: Some(Background::Color(self.colors().background.dark)),
                 border_radius: 4.0.into(),
                 border_width: 1.0,
                 border_color: if *selected {
-                    self.colors.action.base
+                    self.colors().action.base
                 } else {
                     Color::TRANSPARENT
                 },
                 ..Default::default()
             },
             Container::PaneHeader => container::Appearance {
-                background: Some(Background::Color(self.colors.background.darker)),
+                background: Some(Background::Color(self.colors().background.darker)),
                 border_radius: [4.0, 4.0, 0.0, 0.0].into(),
                 border_width: 1.0,
                 border_color: Color::TRANSPARENT,
                 ..Default::default()
             },
             Container::Command { selected } if *selected => container::Appearance {
-                background: Some(Background::Color(self.colors.background.darker)),
+                background: Some(Background::Color(self.colors().background.darker)),
                 border_radius: 3.0.into(),
                 ..Default::default()
             },
@@ -278,21 +215,31 @@ impl container::StyleSheet for Theme {
             },
             Container::Context => container::Appearance {
                 //TODO: Blur background when possible?
-                background: Some(Background::Color(self.colors.background.base)),
+                background: Some(Background::Color(self.colors().background.base)),
                 border_radius: 4.0.into(),
                 border_width: 1.0,
-                border_color: if self.colors.background.is_dark() {
-                    self.colors.background.lighter
+                border_color: if self.colors().background.is_dark() {
+                    self.colors().background.lighter
                 } else {
-                    self.colors.background.darker
+                    self.colors().background.darker
                 },
                 ..Default::default()
             },
             Container::Highlight => container::Appearance {
-                background: Some(Background::Color(self.colors.info.high_alpha)),
+                background: Some(Background::Color(self.colors().info.high_alpha)),
                 border_radius: 0.0.into(),
                 ..Default::default()
             },
+            Container::SemiTransparent => container::Appearance {
+                background: Some(
+                    Color {
+                        a: 0.80,
+                        ..self.colors().background.base
+                    }
+                    .into(),
+                ),
+                ..Default::default()
+            },
         }
     }
 }
@@ -309,6 +256,7 @@ pub enum Button {
         selected: bool,
     },
     Context,
+    Bare,
 }
 
 impl button::StyleSheet for Theme {
@@ -317,19 +265,19 @@ impl button::StyleSheet for Theme {
     fn active(&self, style: &Self::Style) -> button::Appearance {
         match style {
             Button::Default => button::Appearance {
-                background: Some(Background::Color(self.colors.action.high_alpha)),
-                text_color: self.colors.action.base,
+                background: Some(Background::Color(self.colors().action.high_alpha)),
+                text_color: self.colors().action.base,
                 border_radius: 3.0.into(),
                 ..Default::default()
             },
             Button::Secondary => button::Appearance {
-                background: Some(Background::Color(self.colors.text.high_alpha)),
-                text_color: self.colors.text.base,
+                background: Some(Background::Color(self.colors().text.high_alpha)),
+                text_color: self.colors().text.base,
                 border_radius: 3.0.into(),
                 ..Default::default()
             },
             Button::SideMenu { selected } if *selected => button::Appearance {
-                background: Some(Background::Color(self.colors.background.darker)),
+                background: Some(Background::Color(self.colors().background.darker)),
                 border_radius: 3.0.into(),
                 ..Default::default()
             },
@@ -339,18 +287,18 @@ impl button::StyleSheet for Theme {
                 ..Default::default()
             },
             Button::Pane { selected } if *selected => button::Appearance {
-                background: Some(Background::Color(self.colors.action.med_alpha)),
-                border_color: self.colors.action.low_alpha,
+                background: Some(Background::Color(self.colors().action.med_alpha)),
+                border_color: self.colors().action.low_alpha,
                 border_width: 1.0,
                 border_radius: 3.0.into(),
                 ..Default::default()
             },
             Button::Pane { .. } => button::Appearance {
-                background: Some(Background::Color(self.colors.background.dark)),
-                border_color: if self.colors.background.is_dark() {
-                    self.colors.background.lightest
+                background: Some(Background::Color(self.colors().background.dark)),
+                border_color: if self.colors().background.is_dark() {
+                    self.colors().background.lightest
                 } else {
-                    self.colors.background.darkest
+                    self.colors().background.darkest
                 },
                 border_width: 1.0,
                 border_radius: 3.0.into(),
@@ -361,6 +309,10 @@ impl button::StyleSheet for Theme {
                 border_radius: 4.0.into(),
                 ..Default::default()
             },
+            Button::Bare => button::Appearance {
+                background: Some(Background::Color(Color::TRANSPARENT)),
+                ..Default::default()
+            },
         }
     }
 
@@ -372,6 +324,7 @@ impl button::StyleSheet for Theme {
             Button::SideMenu { selected: _ } => button::Appearance { ..active },
             Button::Pane { selected: _ } => button::Appearance { ..active },
             Button::Context => button::Appearance { ..active },
+            Button::Bare => button::Appearance { ..active },
         }
     }
 
@@ -380,41 +333,42 @@ impl button::StyleSheet for Theme {
 
         match style {
             Button::Default => button::Appearance {
-                background: Some(Background::Color(self.colors.action.med_alpha)),
-                text_color: self.colors.action.base,
+                background: Some(Background::Color(self.colors().action.med_alpha)),
+                text_color: self.colors().action.base,
                 border_radius: 3.0.into(),
                 ..Default::default()
             },
             Button::Secondary => button::Appearance {
-                background: Some(Background::Color(self.colors.text.med_alpha)),
-                text_color: self.colors.text.base,
+                background: Some(Background::Color(self.colors().text.med_alpha)),
+                text_color: self.colors().text.base,
                 border_radius: 3.0.into(),
                 ..Default::default()
             },
             Button::SideMenu { selected } if *selected => button::Appearance {
-                background: Some(Background::Color(self.colors.background.darkest)),
+                background: Some(Background::Color(self.colors().background.darkest)),
                 ..active
             },
             Button::SideMenu { .. } => button::Appearance {
-                background: Some(Background::Color(self.colors.background.dark)),
+                background: Some(Background::Color(self.colors().background.dark)),
                 ..active
             },
             Button::Pane { selected } if *selected => button::Appearance {
-                background: Some(Background::Color(self.colors.action.high_alpha)),
+                background: Some(Background::Color(self.colors().action.high_alpha)),
                 ..active
             },
             Button::Pane { .. } => button::Appearance {
-                background: Some(Background::Color(if self.colors.background.is_dark() {
-                    self.colors.background.light
+                background: Some(Background::Color(if self.colors().background.is_dark() {
+                    self.colors().background.light
                 } else {
-                    self.colors.background.darker
+                    self.colors().background.darker
                 })),
                 ..active
             },
             Button::Context => button::Appearance {
-                background: Some(Background::Color(self.colors.background.darker)),
+                background: Some(Background::Color(self.colors().background.darker)),
                 ..active
             },
+            Button::Bare => button::Appearance { ..active },
         }
     }
 
@@ -448,12 +402,12 @@ impl scrollable::StyleSheet for Theme {
     fn active(&self, style: &Self::Style) -> scrollable::Scrollbar {
         match style {
             Scrollable::Default => scrollable::Scrollbar {
-                background: Some(Background::Color(self.colors.background.darker)),
+                background: Some(Background::Color(self.colors().background.darker)),
                 border_radius: 8.0.into(),
                 border_width: 1.0,
                 border_color: Color::TRANSPARENT,
                 scroller: scrollable::Scroller {
-                    color: self.colors.background.darkest,
+                    color: self.colors().background.darkest,
                     border_radius: 8.0.into(),
                     border_width: 0.0,
                     border_color: Color::TRANSPARENT,
@@ -499,9 +453,9 @@ impl pane_grid::StyleSheet for Theme {
     fn hovered_region(&self, style: &Self::Style) -> pane_grid::Appearance {
         match style {
             PaneGrid::Default => pane_grid::Appearance {
-                background: Background::Color(self.colors.action.high_alpha),
+                background: Background::Color(self.colors().action.high_alpha),
                 border_width: 1.0,
-                border_color: self.colors.action.base,
+                border_color: self.colors().action.base,
                 border_radius: 4.0.into(),
             },
         }
@@ -510,7 +464,7 @@ impl pane_grid::StyleSheet for Theme {
     fn picked_split(&self, style: &Self::Style) -> Option<pane_grid::Line> {
         match style {
             PaneGrid::Default => Some(pane_grid::Line {
-                color: self.colors.action.base,
+                color: self.colors().action.base,
                 width: 4.0,
             }),
         }
@@ -519,7 +473,7 @@ impl pane_grid::StyleSheet for Theme {
     fn hovered_split(&self, style: &Self::Style) -> Option<pane_grid::Line> {
         match style {
             PaneGrid::Default => Some(pane_grid::Line {
-                color: self.colors.action.base,
+                color: self.colors().action.base,
                 width: 4.0,
             }),
         }
@@ -539,16 +493,16 @@ impl text_input::StyleSheet for Theme {
     fn active(&self, style: &Self::Style) -> text_input::Appearance {
         match style {
             TextInput::Default => text_input::Appearance {
-                background: Background::Color(self.colors.background.darker),
+                background: Background::Color(self.colors().background.darker),
                 border_radius: 4.0.into(),
                 border_width: 0.0,
                 border_color: Color::TRANSPARENT,
                 // XXX Not currently displayed in application.
-                icon_color: self.colors.info.base,
+                icon_color: self.colors().info.base,
             },
             TextInput::Error => text_input::Appearance {
                 border_width: 1.0,
-                border_color: self.colors.error.base,
+                border_color: self.colors().error.base,
                 ..self.active(&TextInput::Default)
             },
         }
@@ -564,37 +518,37 @@ impl text_input::StyleSheet for Theme {
 
     fn selection_color(&self, style: &Self::Style) -> Color {
         match style {
-            TextInput::Default | TextInput::Error => self.colors.accent.high_alpha,
+            TextInput::Default | TextInput::Error => self.colors().accent.high_alpha,
         }
     }
 
     fn placeholder_color(&self, style: &Self::Style) -> Color {
         match style {
-            TextInput::Default | TextInput::Error => self.colors.text.low_alpha,
+            TextInput::Default | TextInput::Error => self.colors().text.low_alpha,
         }
     }
 
     fn value_color(&self, style: &Self::Style) -> Color {
         match style {
-            TextInput::Default | TextInput::Error => self.colors.text.base,
+            TextInput::Default | TextInput::Error => self.colors().text.base,
         }
     }
 
     fn disabled_color(&self, style: &Self::Style) -> Color {
         match style {
-            TextInput::Default | TextInput::Error => self.colors.text.base,
+            TextInput::Default | TextInput::Error => self.colors().text.base,
         }
     }
 
     fn disabled(&self, style: &Self::Style) -> text_input::Appearance {
         match style {
             TextInput::Default | TextInput::Error => text_input::Appearance {
-                background: Background::Color(self.colors.background.light),
+                background: Background::Color(self.colors().background.light),
                 border_radius: 0.0.into(),
                 border_width: 0.0,
                 border_color: Color::TRANSPARENT,
                 // XXX Not currently displayed in application.
-                icon_color: self.colors.info.base,
+                icon_color: self.colors().info.base,
             },
         }
     }
@@ -605,7 +559,7 @@ impl selectable_text::StyleSheet for Theme {
 
     fn appearance(&self, style: &Self::Style) -> selectable_text::Appearance {
         let color = <Theme as text::StyleSheet>::appearance(self, style.clone()).color;
-        let selection_color = self.colors.accent.high_alpha;
+        let selection_color = self.colors().accent.high_alpha;
 
         selectable_text::Appearance {
             color,
@@ -613,3 +567,63 @@ impl selectable_text::StyleSheet for Theme {
         }
     }
 }
+
+#[derive(Debug, Clone, Default)]
+pub enum Menu {
+    #[default]
+    Default,
+    ComboBox,
+}
+
+impl overlay::menu::StyleSheet for Theme {
+    type Style = Menu;
+
+    fn appearance(&self, style: &Self::Style) -> overlay::menu::Appearance {
+        match style {
+            Menu::Default => overlay::menu::Appearance {
+                text_color: self.colors().text.base,
+                background: Background::Color(self.colors().background.base),
+                border_width: 1.0,
+                border_radius: 4.0.into(),
+                border_color: self.colors().action.base,
+                selected_text_color: self.colors().text.high_alpha,
+                selected_background: Background::Color(self.colors().background.high_alpha),
+            },
+            Menu::ComboBox => overlay::menu::Appearance {
+                text_color: self.colors().text.base,
+                background: Background::Color(self.colors().background.base),
+                border_width: 1.0,
+                border_radius: 4.0.into(),
+                border_color: if self.colors().background.is_dark() {
+                    self.colors().background.lighter
+                } else {
+                    self.colors().background.darker
+                },
+                selected_text_color: self.colors().text.base,
+                selected_background: Background::Color(self.colors().background.dark),
+            },
+        }
+    }
+}
+
+#[derive(Debug, Clone, Default)]
+pub enum ComboBox {
+    #[default]
+    Default,
+}
+
+impl From<ComboBox> for TextInput {
+    fn from(combo_box: ComboBox) -> Self {
+        match combo_box {
+            ComboBox::Default => TextInput::Default,
+        }
+    }
+}
+
+impl From<ComboBox> for Menu {
+    fn from(combo_box: ComboBox) -> Self {
+        match combo_box {
+            ComboBox::Default => Menu::ComboBox,
+        }
+    }
+}
