@@ -3,6 +3,7 @@ use data::{input, Buffer, Command};
 use iced::advanced::widget::{self, Operation};
 pub use iced::widget::text_input::{focus, move_cursor_to_end};
 use iced::widget::{component, container, row, text, text_input, Component};
+use iced::Rectangle;
 
 use self::completion::Completion;
 use super::{anchored_overlay, key_press, Element, Renderer};
@@ -222,7 +223,7 @@ where
             .or_else(|| state.completion.view(self.input))
             .unwrap_or_else(|| row![].into());
 
-        anchored_overlay(input, overlay)
+        anchored_overlay(input, overlay, anchored_overlay::Anchor::AboveTop, 4.0)
     }
 
     fn operate(&self, state: &mut State, operation: &mut dyn widget::Operation<Message>) {
@@ -256,6 +257,7 @@ pub fn reset<Message: 'static>(id: impl Into<widget::Id>) -> iced::Command<Messa
         fn container(
             &mut self,
             _id: Option<&widget::Id>,
+            _bounds: Rectangle,
             operate_on_children: &mut dyn FnMut(&mut dyn Operation<T>),
         ) {
             operate_on_children(self)
